# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from lastmile import Lastmile, AsyncLastmile
from tests.utils import assert_matches_type
from lastmile.types import (
    ProjectGetResponse,
    ProjectListResponse,
    ProjectCreateResponse,
    ProjectDefaultResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestProjects:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Lastmile) -> None:
        project = client.projects.create(
            name="name",
        )
        assert_matches_type(ProjectCreateResponse, project, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Lastmile) -> None:
        project = client.projects.create(
            name="name",
            description="description",
            organization_id="organizationId",
        )
        assert_matches_type(ProjectCreateResponse, project, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Lastmile) -> None:
        response = client.projects.with_raw_response.create(
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        project = response.parse()
        assert_matches_type(ProjectCreateResponse, project, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Lastmile) -> None:
        with client.projects.with_streaming_response.create(
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            project = response.parse()
            assert_matches_type(ProjectCreateResponse, project, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_list(self, client: Lastmile) -> None:
        project = client.projects.list()
        assert_matches_type(ProjectListResponse, project, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Lastmile) -> None:
        project = client.projects.list(
            filters={
                "can_contribute": True,
                "organization_id": "organizationId",
                "query": "query",
            },
        )
        assert_matches_type(ProjectListResponse, project, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Lastmile) -> None:
        response = client.projects.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        project = response.parse()
        assert_matches_type(ProjectListResponse, project, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Lastmile) -> None:
        with client.projects.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            project = response.parse()
            assert_matches_type(ProjectListResponse, project, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_default(self, client: Lastmile) -> None:
        project = client.projects.default()
        assert_matches_type(ProjectDefaultResponse, project, path=["response"])

    @parametrize
    def test_method_default_with_all_params(self, client: Lastmile) -> None:
        project = client.projects.default(
            organization_id="organizationId",
        )
        assert_matches_type(ProjectDefaultResponse, project, path=["response"])

    @parametrize
    def test_raw_response_default(self, client: Lastmile) -> None:
        response = client.projects.with_raw_response.default()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        project = response.parse()
        assert_matches_type(ProjectDefaultResponse, project, path=["response"])

    @parametrize
    def test_streaming_response_default(self, client: Lastmile) -> None:
        with client.projects.with_streaming_response.default() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            project = response.parse()
            assert_matches_type(ProjectDefaultResponse, project, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_get(self, client: Lastmile) -> None:
        project = client.projects.get(
            id="id",
        )
        assert_matches_type(ProjectGetResponse, project, path=["response"])

    @parametrize
    def test_raw_response_get(self, client: Lastmile) -> None:
        response = client.projects.with_raw_response.get(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        project = response.parse()
        assert_matches_type(ProjectGetResponse, project, path=["response"])

    @parametrize
    def test_streaming_response_get(self, client: Lastmile) -> None:
        with client.projects.with_streaming_response.get(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            project = response.parse()
            assert_matches_type(ProjectGetResponse, project, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncProjects:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncLastmile) -> None:
        project = await async_client.projects.create(
            name="name",
        )
        assert_matches_type(ProjectCreateResponse, project, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncLastmile) -> None:
        project = await async_client.projects.create(
            name="name",
            description="description",
            organization_id="organizationId",
        )
        assert_matches_type(ProjectCreateResponse, project, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncLastmile) -> None:
        response = await async_client.projects.with_raw_response.create(
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        project = await response.parse()
        assert_matches_type(ProjectCreateResponse, project, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncLastmile) -> None:
        async with async_client.projects.with_streaming_response.create(
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            project = await response.parse()
            assert_matches_type(ProjectCreateResponse, project, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_list(self, async_client: AsyncLastmile) -> None:
        project = await async_client.projects.list()
        assert_matches_type(ProjectListResponse, project, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncLastmile) -> None:
        project = await async_client.projects.list(
            filters={
                "can_contribute": True,
                "organization_id": "organizationId",
                "query": "query",
            },
        )
        assert_matches_type(ProjectListResponse, project, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncLastmile) -> None:
        response = await async_client.projects.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        project = await response.parse()
        assert_matches_type(ProjectListResponse, project, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncLastmile) -> None:
        async with async_client.projects.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            project = await response.parse()
            assert_matches_type(ProjectListResponse, project, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_default(self, async_client: AsyncLastmile) -> None:
        project = await async_client.projects.default()
        assert_matches_type(ProjectDefaultResponse, project, path=["response"])

    @parametrize
    async def test_method_default_with_all_params(self, async_client: AsyncLastmile) -> None:
        project = await async_client.projects.default(
            organization_id="organizationId",
        )
        assert_matches_type(ProjectDefaultResponse, project, path=["response"])

    @parametrize
    async def test_raw_response_default(self, async_client: AsyncLastmile) -> None:
        response = await async_client.projects.with_raw_response.default()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        project = await response.parse()
        assert_matches_type(ProjectDefaultResponse, project, path=["response"])

    @parametrize
    async def test_streaming_response_default(self, async_client: AsyncLastmile) -> None:
        async with async_client.projects.with_streaming_response.default() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            project = await response.parse()
            assert_matches_type(ProjectDefaultResponse, project, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_get(self, async_client: AsyncLastmile) -> None:
        project = await async_client.projects.get(
            id="id",
        )
        assert_matches_type(ProjectGetResponse, project, path=["response"])

    @parametrize
    async def test_raw_response_get(self, async_client: AsyncLastmile) -> None:
        response = await async_client.projects.with_raw_response.get(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        project = await response.parse()
        assert_matches_type(ProjectGetResponse, project, path=["response"])

    @parametrize
    async def test_streaming_response_get(self, async_client: AsyncLastmile) -> None:
        async with async_client.projects.with_streaming_response.get(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            project = await response.parse()
            assert_matches_type(ProjectGetResponse, project, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any, Union, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    NOT_GIVEN,
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
)
from ._utils import is_given, get_async_library
from ._compat import cached_property
from ._version import __version__
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import LastmileError, APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

if TYPE_CHECKING:
    from .resources import datasets, projects, evaluation, experiments, fine_tune_jobs, label_dataset_jobs
    from .resources.datasets import DatasetsResource, AsyncDatasetsResource
    from .resources.projects import ProjectsResource, AsyncProjectsResource
    from .resources.evaluation import EvaluationResource, AsyncEvaluationResource
    from .resources.experiments import ExperimentsResource, AsyncExperimentsResource
    from .resources.fine_tune_jobs import FineTuneJobsResource, AsyncFineTuneJobsResource
    from .resources.label_dataset_jobs import LabelDatasetJobsResource, AsyncLabelDatasetJobsResource

__all__ = [
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "Lastmile",
    "AsyncLastmile",
    "Client",
    "AsyncClient",
]


class Lastmile(SyncAPIClient):
    # client options
    bearer_token: str

    def __init__(
        self,
        *,
        bearer_token: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Lastmile client instance.

        This automatically infers the `bearer_token` argument from the `LASTMILE_API_TOKEN` environment variable if it is not provided.
        """
        if bearer_token is None:
            bearer_token = os.environ.get("LASTMILE_API_TOKEN")
        if bearer_token is None:
            raise LastmileError(
                "The bearer_token client option must be set either by passing bearer_token to the client or by setting the LASTMILE_API_TOKEN environment variable"
            )
        self.bearer_token = bearer_token

        if base_url is None:
            base_url = os.environ.get("LASTMILE_BASE_URL")
        if base_url is None:
            base_url = f"https://lastmileai.dev"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def projects(self) -> ProjectsResource:
        from .resources.projects import ProjectsResource

        return ProjectsResource(self)

    @cached_property
    def experiments(self) -> ExperimentsResource:
        from .resources.experiments import ExperimentsResource

        return ExperimentsResource(self)

    @cached_property
    def datasets(self) -> DatasetsResource:
        from .resources.datasets import DatasetsResource

        return DatasetsResource(self)

    @cached_property
    def evaluation(self) -> EvaluationResource:
        from .resources.evaluation import EvaluationResource

        return EvaluationResource(self)

    @cached_property
    def fine_tune_jobs(self) -> FineTuneJobsResource:
        from .resources.fine_tune_jobs import FineTuneJobsResource

        return FineTuneJobsResource(self)

    @cached_property
    def label_dataset_jobs(self) -> LabelDatasetJobsResource:
        from .resources.label_dataset_jobs import LabelDatasetJobsResource

        return LabelDatasetJobsResource(self)

    @cached_property
    def with_raw_response(self) -> LastmileWithRawResponse:
        return LastmileWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LastmileWithStreamedResponse:
        return LastmileWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        bearer_token = self.bearer_token
        return {"Authorization": f"Bearer {bearer_token}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        bearer_token: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            bearer_token=bearer_token or self.bearer_token,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncLastmile(AsyncAPIClient):
    # client options
    bearer_token: str

    def __init__(
        self,
        *,
        bearer_token: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncLastmile client instance.

        This automatically infers the `bearer_token` argument from the `LASTMILE_API_TOKEN` environment variable if it is not provided.
        """
        if bearer_token is None:
            bearer_token = os.environ.get("LASTMILE_API_TOKEN")
        if bearer_token is None:
            raise LastmileError(
                "The bearer_token client option must be set either by passing bearer_token to the client or by setting the LASTMILE_API_TOKEN environment variable"
            )
        self.bearer_token = bearer_token

        if base_url is None:
            base_url = os.environ.get("LASTMILE_BASE_URL")
        if base_url is None:
            base_url = f"https://lastmileai.dev"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def projects(self) -> AsyncProjectsResource:
        from .resources.projects import AsyncProjectsResource

        return AsyncProjectsResource(self)

    @cached_property
    def experiments(self) -> AsyncExperimentsResource:
        from .resources.experiments import AsyncExperimentsResource

        return AsyncExperimentsResource(self)

    @cached_property
    def datasets(self) -> AsyncDatasetsResource:
        from .resources.datasets import AsyncDatasetsResource

        return AsyncDatasetsResource(self)

    @cached_property
    def evaluation(self) -> AsyncEvaluationResource:
        from .resources.evaluation import AsyncEvaluationResource

        return AsyncEvaluationResource(self)

    @cached_property
    def fine_tune_jobs(self) -> AsyncFineTuneJobsResource:
        from .resources.fine_tune_jobs import AsyncFineTuneJobsResource

        return AsyncFineTuneJobsResource(self)

    @cached_property
    def label_dataset_jobs(self) -> AsyncLabelDatasetJobsResource:
        from .resources.label_dataset_jobs import AsyncLabelDatasetJobsResource

        return AsyncLabelDatasetJobsResource(self)

    @cached_property
    def with_raw_response(self) -> AsyncLastmileWithRawResponse:
        return AsyncLastmileWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLastmileWithStreamedResponse:
        return AsyncLastmileWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        bearer_token = self.bearer_token
        return {"Authorization": f"Bearer {bearer_token}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        bearer_token: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            bearer_token=bearer_token or self.bearer_token,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class LastmileWithRawResponse:
    _client: Lastmile

    def __init__(self, client: Lastmile) -> None:
        self._client = client

    @cached_property
    def projects(self) -> projects.ProjectsResourceWithRawResponse:
        from .resources.projects import ProjectsResourceWithRawResponse

        return ProjectsResourceWithRawResponse(self._client.projects)

    @cached_property
    def experiments(self) -> experiments.ExperimentsResourceWithRawResponse:
        from .resources.experiments import ExperimentsResourceWithRawResponse

        return ExperimentsResourceWithRawResponse(self._client.experiments)

    @cached_property
    def datasets(self) -> datasets.DatasetsResourceWithRawResponse:
        from .resources.datasets import DatasetsResourceWithRawResponse

        return DatasetsResourceWithRawResponse(self._client.datasets)

    @cached_property
    def evaluation(self) -> evaluation.EvaluationResourceWithRawResponse:
        from .resources.evaluation import EvaluationResourceWithRawResponse

        return EvaluationResourceWithRawResponse(self._client.evaluation)

    @cached_property
    def fine_tune_jobs(self) -> fine_tune_jobs.FineTuneJobsResourceWithRawResponse:
        from .resources.fine_tune_jobs import FineTuneJobsResourceWithRawResponse

        return FineTuneJobsResourceWithRawResponse(self._client.fine_tune_jobs)

    @cached_property
    def label_dataset_jobs(self) -> label_dataset_jobs.LabelDatasetJobsResourceWithRawResponse:
        from .resources.label_dataset_jobs import LabelDatasetJobsResourceWithRawResponse

        return LabelDatasetJobsResourceWithRawResponse(self._client.label_dataset_jobs)


class AsyncLastmileWithRawResponse:
    _client: AsyncLastmile

    def __init__(self, client: AsyncLastmile) -> None:
        self._client = client

    @cached_property
    def projects(self) -> projects.AsyncProjectsResourceWithRawResponse:
        from .resources.projects import AsyncProjectsResourceWithRawResponse

        return AsyncProjectsResourceWithRawResponse(self._client.projects)

    @cached_property
    def experiments(self) -> experiments.AsyncExperimentsResourceWithRawResponse:
        from .resources.experiments import AsyncExperimentsResourceWithRawResponse

        return AsyncExperimentsResourceWithRawResponse(self._client.experiments)

    @cached_property
    def datasets(self) -> datasets.AsyncDatasetsResourceWithRawResponse:
        from .resources.datasets import AsyncDatasetsResourceWithRawResponse

        return AsyncDatasetsResourceWithRawResponse(self._client.datasets)

    @cached_property
    def evaluation(self) -> evaluation.AsyncEvaluationResourceWithRawResponse:
        from .resources.evaluation import AsyncEvaluationResourceWithRawResponse

        return AsyncEvaluationResourceWithRawResponse(self._client.evaluation)

    @cached_property
    def fine_tune_jobs(self) -> fine_tune_jobs.AsyncFineTuneJobsResourceWithRawResponse:
        from .resources.fine_tune_jobs import AsyncFineTuneJobsResourceWithRawResponse

        return AsyncFineTuneJobsResourceWithRawResponse(self._client.fine_tune_jobs)

    @cached_property
    def label_dataset_jobs(self) -> label_dataset_jobs.AsyncLabelDatasetJobsResourceWithRawResponse:
        from .resources.label_dataset_jobs import AsyncLabelDatasetJobsResourceWithRawResponse

        return AsyncLabelDatasetJobsResourceWithRawResponse(self._client.label_dataset_jobs)


class LastmileWithStreamedResponse:
    _client: Lastmile

    def __init__(self, client: Lastmile) -> None:
        self._client = client

    @cached_property
    def projects(self) -> projects.ProjectsResourceWithStreamingResponse:
        from .resources.projects import ProjectsResourceWithStreamingResponse

        return ProjectsResourceWithStreamingResponse(self._client.projects)

    @cached_property
    def experiments(self) -> experiments.ExperimentsResourceWithStreamingResponse:
        from .resources.experiments import ExperimentsResourceWithStreamingResponse

        return ExperimentsResourceWithStreamingResponse(self._client.experiments)

    @cached_property
    def datasets(self) -> datasets.DatasetsResourceWithStreamingResponse:
        from .resources.datasets import DatasetsResourceWithStreamingResponse

        return DatasetsResourceWithStreamingResponse(self._client.datasets)

    @cached_property
    def evaluation(self) -> evaluation.EvaluationResourceWithStreamingResponse:
        from .resources.evaluation import EvaluationResourceWithStreamingResponse

        return EvaluationResourceWithStreamingResponse(self._client.evaluation)

    @cached_property
    def fine_tune_jobs(self) -> fine_tune_jobs.FineTuneJobsResourceWithStreamingResponse:
        from .resources.fine_tune_jobs import FineTuneJobsResourceWithStreamingResponse

        return FineTuneJobsResourceWithStreamingResponse(self._client.fine_tune_jobs)

    @cached_property
    def label_dataset_jobs(self) -> label_dataset_jobs.LabelDatasetJobsResourceWithStreamingResponse:
        from .resources.label_dataset_jobs import LabelDatasetJobsResourceWithStreamingResponse

        return LabelDatasetJobsResourceWithStreamingResponse(self._client.label_dataset_jobs)


class AsyncLastmileWithStreamedResponse:
    _client: AsyncLastmile

    def __init__(self, client: AsyncLastmile) -> None:
        self._client = client

    @cached_property
    def projects(self) -> projects.AsyncProjectsResourceWithStreamingResponse:
        from .resources.projects import AsyncProjectsResourceWithStreamingResponse

        return AsyncProjectsResourceWithStreamingResponse(self._client.projects)

    @cached_property
    def experiments(self) -> experiments.AsyncExperimentsResourceWithStreamingResponse:
        from .resources.experiments import AsyncExperimentsResourceWithStreamingResponse

        return AsyncExperimentsResourceWithStreamingResponse(self._client.experiments)

    @cached_property
    def datasets(self) -> datasets.AsyncDatasetsResourceWithStreamingResponse:
        from .resources.datasets import AsyncDatasetsResourceWithStreamingResponse

        return AsyncDatasetsResourceWithStreamingResponse(self._client.datasets)

    @cached_property
    def evaluation(self) -> evaluation.AsyncEvaluationResourceWithStreamingResponse:
        from .resources.evaluation import AsyncEvaluationResourceWithStreamingResponse

        return AsyncEvaluationResourceWithStreamingResponse(self._client.evaluation)

    @cached_property
    def fine_tune_jobs(self) -> fine_tune_jobs.AsyncFineTuneJobsResourceWithStreamingResponse:
        from .resources.fine_tune_jobs import AsyncFineTuneJobsResourceWithStreamingResponse

        return AsyncFineTuneJobsResourceWithStreamingResponse(self._client.fine_tune_jobs)

    @cached_property
    def label_dataset_jobs(self) -> label_dataset_jobs.AsyncLabelDatasetJobsResourceWithStreamingResponse:
        from .resources.label_dataset_jobs import AsyncLabelDatasetJobsResourceWithStreamingResponse

        return AsyncLabelDatasetJobsResourceWithStreamingResponse(self._client.label_dataset_jobs)


Client = Lastmile

AsyncClient = AsyncLastmile



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["EvaluationGetMetricParams", "Metric"]


class EvaluationGetMetricParams(TypedDict, total=False):
    metric: Required[Metric]


class Metric(TypedDict, total=False):
    id: str

    deployment_status: Annotated[
        Literal[
            "MODEL_DEPLOYMENT_STATUS_UNSPECIFIED",
            "MODEL_DEPLOYMENT_STATUS_PENDING",
            "MODEL_DEPLOYMENT_STATUS_ONLINE",
            "MODEL_DEPLOYMENT_STATUS_OFFLINE",
            "MODEL_DEPLOYMENT_STATUS_PAUSED",
        ],
        PropertyInfo(alias="deploymentStatus"),
    ]

    description: str

    name: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["ExperimentDeleteResponse"]


class ExperimentDeleteResponse(BaseModel):
    id: str
    """ID of the deleted experiment"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from lastmile import Lastmile, AsyncLastmile
from tests.utils import assert_matches_type
from lastmile.types import (
    ExperimentGetResponse,
    ExperimentListResponse,
    ExperimentCreateResponse,
    ExperimentDeleteResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestExperiments:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Lastmile) -> None:
        experiment = client.experiments.create(
            name="name",
            project_id="projectId",
        )
        assert_matches_type(ExperimentCreateResponse, experiment, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Lastmile) -> None:
        experiment = client.experiments.create(
            name="name",
            project_id="projectId",
            description="description",
            metadata={"fields": {"foo": {"foo": "bar"}}},
        )
        assert_matches_type(ExperimentCreateResponse, experiment, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Lastmile) -> None:
        response = client.experiments.with_raw_response.create(
            name="name",
            project_id="projectId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        experiment = response.parse()
        assert_matches_type(ExperimentCreateResponse, experiment, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Lastmile) -> None:
        with client.experiments.with_streaming_response.create(
            name="name",
            project_id="projectId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            experiment = response.parse()
            assert_matches_type(ExperimentCreateResponse, experiment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_list(self, client: Lastmile) -> None:
        experiment = client.experiments.list()
        assert_matches_type(ExperimentListResponse, experiment, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Lastmile) -> None:
        experiment = client.experiments.list(
            filters={
                "project_id": "projectId",
                "query": "query",
            },
            page_index=0,
            page_size=0,
        )
        assert_matches_type(ExperimentListResponse, experiment, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Lastmile) -> None:
        response = client.experiments.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        experiment = response.parse()
        assert_matches_type(ExperimentListResponse, experiment, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Lastmile) -> None:
        with client.experiments.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            experiment = response.parse()
            assert_matches_type(ExperimentListResponse, experiment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: Lastmile) -> None:
        experiment = client.experiments.delete(
            id="id",
        )
        assert_matches_type(ExperimentDeleteResponse, experiment, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: Lastmile) -> None:
        response = client.experiments.with_raw_response.delete(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        experiment = response.parse()
        assert_matches_type(ExperimentDeleteResponse, experiment, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: Lastmile) -> None:
        with client.experiments.with_streaming_response.delete(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            experiment = response.parse()
            assert_matches_type(ExperimentDeleteResponse, experiment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_get(self, client: Lastmile) -> None:
        experiment = client.experiments.get(
            id="id",
        )
        assert_matches_type(ExperimentGetResponse, experiment, path=["response"])

    @parametrize
    def test_raw_response_get(self, client: Lastmile) -> None:
        response = client.experiments.with_raw_response.get(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        experiment = response.parse()
        assert_matches_type(ExperimentGetResponse, experiment, path=["response"])

    @parametrize
    def test_streaming_response_get(self, client: Lastmile) -> None:
        with client.experiments.with_streaming_response.get(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            experiment = response.parse()
            assert_matches_type(ExperimentGetResponse, experiment, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncExperiments:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncLastmile) -> None:
        experiment = await async_client.experiments.create(
            name="name",
            project_id="projectId",
        )
        assert_matches_type(ExperimentCreateResponse, experiment, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncLastmile) -> None:
        experiment = await async_client.experiments.create(
            name="name",
            project_id="projectId",
            description="description",
            metadata={"fields": {"foo": {"foo": "bar"}}},
        )
        assert_matches_type(ExperimentCreateResponse, experiment, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncLastmile) -> None:
        response = await async_client.experiments.with_raw_response.create(
            name="name",
            project_id="projectId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        experiment = await response.parse()
        assert_matches_type(ExperimentCreateResponse, experiment, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncLastmile) -> None:
        async with async_client.experiments.with_streaming_response.create(
            name="name",
            project_id="projectId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            experiment = await response.parse()
            assert_matches_type(ExperimentCreateResponse, experiment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_list(self, async_client: AsyncLastmile) -> None:
        experiment = await async_client.experiments.list()
        assert_matches_type(ExperimentListResponse, experiment, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncLastmile) -> None:
        experiment = await async_client.experiments.list(
            filters={
                "project_id": "projectId",
                "query": "query",
            },
            page_index=0,
            page_size=0,
        )
        assert_matches_type(ExperimentListResponse, experiment, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncLastmile) -> None:
        response = await async_client.experiments.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        experiment = await response.parse()
        assert_matches_type(ExperimentListResponse, experiment, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncLastmile) -> None:
        async with async_client.experiments.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            experiment = await response.parse()
            assert_matches_type(ExperimentListResponse, experiment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncLastmile) -> None:
        experiment = await async_client.experiments.delete(
            id="id",
        )
        assert_matches_type(ExperimentDeleteResponse, experiment, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncLastmile) -> None:
        response = await async_client.experiments.with_raw_response.delete(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        experiment = await response.parse()
        assert_matches_type(ExperimentDeleteResponse, experiment, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncLastmile) -> None:
        async with async_client.experiments.with_streaming_response.delete(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            experiment = await response.parse()
            assert_matches_type(ExperimentDeleteResponse, experiment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_get(self, async_client: AsyncLastmile) -> None:
        experiment = await async_client.experiments.get(
            id="id",
        )
        assert_matches_type(ExperimentGetResponse, experiment, path=["response"])

    @parametrize
    async def test_raw_response_get(self, async_client: AsyncLastmile) -> None:
        response = await async_client.experiments.with_raw_response.get(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        experiment = await response.parse()
        assert_matches_type(ExperimentGetResponse, experiment, path=["response"])

    @parametrize
    async def test_streaming_response_get(self, async_client: AsyncLastmile) -> None:
        async with async_client.experiments.with_streaming_response.get(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            experiment = await response.parse()
            assert_matches_type(ExperimentGetResponse, experiment, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .datasets import (
    DatasetsResource,
    AsyncDatasetsResource,
    DatasetsResourceWithRawResponse,
    AsyncDatasetsResourceWithRawResponse,
    DatasetsResourceWithStreamingResponse,
    AsyncDatasetsResourceWithStreamingResponse,
)
from .projects import (
    ProjectsResource,
    AsyncProjectsResource,
    ProjectsResourceWithRawResponse,
    AsyncProjectsResourceWithRawResponse,
    ProjectsResourceWithStreamingResponse,
    AsyncProjectsResourceWithStreamingResponse,
)
from .evaluation import (
    EvaluationResource,
    AsyncEvaluationResource,
    EvaluationResourceWithRawResponse,
    AsyncEvaluationResourceWithRawResponse,
    EvaluationResourceWithStreamingResponse,
    AsyncEvaluationResourceWithStreamingResponse,
)
from .experiments import (
    ExperimentsResource,
    AsyncExperimentsResource,
    ExperimentsResourceWithRawResponse,
    AsyncExperimentsResourceWithRawResponse,
    ExperimentsResourceWithStreamingResponse,
    AsyncExperimentsResourceWithStreamingResponse,
)
from .fine_tune_jobs import (
    FineTuneJobsResource,
    AsyncFineTuneJobsResource,
    FineTuneJobsResourceWithRawResponse,
    AsyncFineTuneJobsResourceWithRawResponse,
    FineTuneJobsResourceWithStreamingResponse,
    AsyncFineTuneJobsResourceWithStreamingResponse,
)
from .label_dataset_jobs import (
    LabelDatasetJobsResource,
    AsyncLabelDatasetJobsResource,
    LabelDatasetJobsResourceWithRawResponse,
    AsyncLabelDatasetJobsResourceWithRawResponse,
    LabelDatasetJobsResourceWithStreamingResponse,
    AsyncLabelDatasetJobsResourceWithStreamingResponse,
)

__all__ = [
    "ProjectsResource",
    "AsyncProjectsResource",
    "ProjectsResourceWithRawResponse",
    "AsyncProjectsResourceWithRawResponse",
    "ProjectsResourceWithStreamingResponse",
    "AsyncProjectsResourceWithStreamingResponse",
    "ExperimentsResource",
    "AsyncExperimentsResource",
    "ExperimentsResourceWithRawResponse",
    "AsyncExperimentsResourceWithRawResponse",
    "ExperimentsResourceWithStreamingResponse",
    "AsyncExperimentsResourceWithStreamingResponse",
    "DatasetsResource",
    "AsyncDatasetsResource",
    "DatasetsResourceWithRawResponse",
    "AsyncDatasetsResourceWithRawResponse",
    "DatasetsResourceWithStreamingResponse",
    "AsyncDatasetsResourceWithStreamingResponse",
    "EvaluationResource",
    "AsyncEvaluationResource",
    "EvaluationResourceWithRawResponse",
    "AsyncEvaluationResourceWithRawResponse",
    "EvaluationResourceWithStreamingResponse",
    "AsyncEvaluationResourceWithStreamingResponse",
    "FineTuneJobsResource",
    "AsyncFineTuneJobsResource",
    "FineTuneJobsResourceWithRawResponse",
    "AsyncFineTuneJobsResourceWithRawResponse",
    "FineTuneJobsResourceWithStreamingResponse",
    "AsyncFineTuneJobsResourceWithStreamingResponse",
    "LabelDatasetJobsResource",
    "AsyncLabelDatasetJobsResource",
    "LabelDatasetJobsResourceWithRawResponse",
    "AsyncLabelDatasetJobsResourceWithRawResponse",
    "LabelDatasetJobsResourceWithStreamingResponse",
    "AsyncLabelDatasetJobsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ..types import (
    dataset_get_params,
    dataset_copy_params,
    dataset_list_params,
    dataset_create_params,
    dataset_delete_params,
    dataset_get_view_params,
    dataset_upload_file_params,
    dataset_get_download_url_params,
    dataset_finalize_file_upload_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.dataset_get_response import DatasetGetResponse
from ..types.dataset_copy_response import DatasetCopyResponse
from ..types.dataset_list_response import DatasetListResponse
from ..types.dataset_create_response import DatasetCreateResponse
from ..types.dataset_delete_response import DatasetDeleteResponse
from ..types.dataset_get_view_response import DatasetGetViewResponse
from ..types.dataset_upload_file_response import DatasetUploadFileResponse
from ..types.dataset_get_download_url_response import DatasetGetDownloadURLResponse
from ..types.dataset_finalize_file_upload_response import DatasetFinalizeFileUploadResponse

__all__ = ["DatasetsResource", "AsyncDatasetsResource"]


class DatasetsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DatasetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/lastmile-ai/lastmile-python#accessing-raw-response-data-eg-headers
        """
        return DatasetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DatasetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/lastmile-ai/lastmile-python#with_streaming_response
        """
        return DatasetsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        description: str | NotGiven = NOT_GIVEN,
        is_active_labels: bool | NotGiven = NOT_GIVEN,
        is_few_shot_examples: bool | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        project_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DatasetCreateResponse:
        """Create a new Dataset.

        Use UploadDatasetFile to upload files to the dataset.

        Args:
          description: Human-readable description of the dataset, if one exists.

          is_few_shot_examples: PseudoLabel job fields.

          name: Human-readable name for the dataset, if one exists.

          project_id: The project to add the new dataset to

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/2/auto_eval/dataset/create",
            body=maybe_transform(
                {
                    "description": description,
                    "is_active_labels": is_active_labels,
                    "is_few_shot_examples": is_few_shot_examples,
                    "name": name,
                    "project_id": project_id,
                },
                dataset_create_params.DatasetCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetCreateResponse,
        )

    def list(
        self,
        *,
        filters: dataset_list_params.Filters | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DatasetListResponse:
        """
        List Datasets.

        Args:
          filters: Filter listed datasets by ALL filters specified

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/2/auto_eval/dataset/list",
            body=maybe_transform({"filters": filters}, dataset_list_params.DatasetListParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetListResponse,
        )

    def delete(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DatasetDeleteResponse:
        """
        Archive a dataset

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/2/auto_eval/dataset/archive",
            body=maybe_transform({"id": id}, dataset_delete_params.DatasetDeleteParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetDeleteResponse,
        )

    def copy(
        self,
        *,
        dataset_id: str,
        description: str | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        project_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DatasetCopyResponse:
        """
        Clone a dataset into a new dataset containing the latest file contents

        Args:
          dataset_id: Dataset to clone

          description: Human-readable description of the dataset. If not provided, will use the name of
              the dataset being cloned

          name: New dataset name. If not provided, will use the name of the dataset being cloned

          project_id: The project to add the new dataset to

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/2/auto_eval/dataset/clone",
            body=maybe_transform(
                {
                    "dataset_id": dataset_id,
                    "description": description,
                    "name": name,
                    "project_id": project_id,
                },
                dataset_copy_params.DatasetCopyParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetCopyResponse,
        )

    def finalize_file_upload(
        self,
        *,
        dataset_id: str,
        s3_presigned_post: dataset_finalize_file_upload_params.S3PresignedPost,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DatasetFinalizeFileUploadResponse:
        """Finalize a Dataset file upload.

        This call should be made after the file has been
        uploaded to the S3 URL returned from UploadDatasetFile.

        Args:
          dataset_id: The ID of the dataset corresponding to the file

          s3_presigned_post: The pre-signed S3 URL where the file was uploadeded

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/2/auto_eval/dataset/finalize_single_file_upload",
            body=maybe_transform(
                {
                    "dataset_id": dataset_id,
                    "s3_presigned_post": s3_presigned_post,
                },
                dataset_finalize_file_upload_params.DatasetFinalizeFileUploadParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetFinalizeFileUploadResponse,
        )

    def get(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DatasetGetResponse:
        """
        Get a Dataset.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/2/auto_eval/dataset/get",
            body=maybe_transform({"id": id}, dataset_get_params.DatasetGetParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetGetResponse,
        )

    def get_download_url(
        self,
        *,
        dataset_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DatasetGetDownloadURLResponse:
        """
        Get a download url for a Dataset.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/2/auto_eval/dataset/get_download_url",
            body=maybe_transform(
                {"dataset_id": dataset_id}, dataset_get_download_url_params.DatasetGetDownloadURLParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetGetDownloadURLResponse,
        )

    def get_view(
        self,
        *,
        dataset_file_id: str,
        dataset_id: str,
        filters: Iterable[dataset_get_view_params.Filter],
        after: int | NotGiven = NOT_GIVEN,
        get_last_page: bool | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        next_page_cursor: str | NotGiven = NOT_GIVEN,
        order_by: str | NotGiven = NOT_GIVEN,
        order_direction: str | NotGiven = NOT_GIVEN,
        previous_page_cursor: str | NotGiven = NOT_GIVEN,
        use_datasets_service: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DatasetGetViewResponse:
        """
        Get a paginated view of the data within a Dataset.

        Args:
          dataset_file_id: The ID of the (pinned) dataset file from which to retrieve content. Requests
              iterating over pages of results are recommended to use this pinned identifier
              after the first page in order to prevent any effects from a dataset changing
              between the queries.

          dataset_id: The ID of the dataset from which to retrieve content. When specified, gets data
              from the current file in the dataset.

          after: Pagination: The index, by row-order, after which to query results.

          limit: Pagination: The maximum number of results to return on this page.

          next_page_cursor: A cursor for the next page in the pagination, if one exists.

          order_by: Column to order results by

          order_direction: Direction to order results ("asc" or "desc")

          previous_page_cursor: A cursor for the previous page in the pagination, if one exists.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/2/auto_eval/dataset/get_view",
            body=maybe_transform(
                {
                    "dataset_file_id": dataset_file_id,
                    "dataset_id": dataset_id,
                    "filters": filters,
                    "after": after,
                    "get_last_page": get_last_page,
                    "limit": limit,
                    "next_page_cursor": next_page_cursor,
                    "order_by": order_by,
                    "order_direction": order_direction,
                    "previous_page_cursor": previous_page_cursor,
                    "use_datasets_service": use_datasets_service,
                },
                dataset_get_view_params.DatasetGetViewParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetGetViewResponse,
        )

    def upload_file(
        self,
        *,
        dataset_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DatasetUploadFileResponse:
        """Initiate a file upload to a Dataset.

        Call FinalizeSingleDatasetFileUpload to
        complete the upload with the presigned URL returned from this call.

        Args:
          dataset_id: The ID of the dataset corresponding to the file.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/2/auto_eval/dataset/upload_file",
            body=maybe_transform({"dataset_id": dataset_id}, dataset_upload_file_params.DatasetUploadFileParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetUploadFileResponse,
        )


class AsyncDatasetsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDatasetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/lastmile-ai/lastmile-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDatasetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDatasetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/lastmile-ai/lastmile-python#with_streaming_response
        """
        return AsyncDatasetsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        description: str | NotGiven = NOT_GIVEN,
        is_active_labels: bool | NotGiven = NOT_GIVEN,
        is_few_shot_examples: bool | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        project_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DatasetCreateResponse:
        """Create a new Dataset.

        Use UploadDatasetFile to upload files to the dataset.

        Args:
          description: Human-readable description of the dataset, if one exists.

          is_few_shot_examples: PseudoLabel job fields.

          name: Human-readable name for the dataset, if one exists.

          project_id: The project to add the new dataset to

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/2/auto_eval/dataset/create",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "is_active_labels": is_active_labels,
                    "is_few_shot_examples": is_few_shot_examples,
                    "name": name,
                    "project_id": project_id,
                },
                dataset_create_params.DatasetCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetCreateResponse,
        )

    async def list(
        self,
        *,
        filters: dataset_list_params.Filters | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DatasetListResponse:
        """
        List Datasets.

        Args:
          filters: Filter listed datasets by ALL filters specified

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/2/auto_eval/dataset/list",
            body=await async_maybe_transform({"filters": filters}, dataset_list_params.DatasetListParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetListResponse,
        )

    async def delete(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DatasetDeleteResponse:
        """
        Archive a dataset

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/2/auto_eval/dataset/archive",
            body=await async_maybe_transform({"id": id}, dataset_delete_params.DatasetDeleteParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetDeleteResponse,
        )

    async def copy(
        self,
        *,
        dataset_id: str,
        description: str | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        project_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DatasetCopyResponse:
        """
        Clone a dataset into a new dataset containing the latest file contents

        Args:
          dataset_id: Dataset to clone

          description: Human-readable description of the dataset. If not provided, will use the name of
              the dataset being cloned

          name: New dataset name. If not provided, will use the name of the dataset being cloned

          project_id: The project to add the new dataset to

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/2/auto_eval/dataset/clone",
            body=await async_maybe_transform(
                {
                    "dataset_id": dataset_id,
                    "description": description,
                    "name": name,
                    "project_id": project_id,
                },
                dataset_copy_params.DatasetCopyParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetCopyResponse,
        )

    async def finalize_file_upload(
        self,
        *,
        dataset_id: str,
        s3_presigned_post: dataset_finalize_file_upload_params.S3PresignedPost,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DatasetFinalizeFileUploadResponse:
        """Finalize a Dataset file upload.

        This call should be made after the file has been
        uploaded to the S3 URL returned from UploadDatasetFile.

        Args:
          dataset_id: The ID of the dataset corresponding to the file

          s3_presigned_post: The pre-signed S3 URL where the file was uploadeded

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/2/auto_eval/dataset/finalize_single_file_upload",
            body=await async_maybe_transform(
                {
                    "dataset_id": dataset_id,
                    "s3_presigned_post": s3_presigned_post,
                },
                dataset_finalize_file_upload_params.DatasetFinalizeFileUploadParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetFinalizeFileUploadResponse,
        )

    async def get(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DatasetGetResponse:
        """
        Get a Dataset.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/2/auto_eval/dataset/get",
            body=await async_maybe_transform({"id": id}, dataset_get_params.DatasetGetParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetGetResponse,
        )

    async def get_download_url(
        self,
        *,
        dataset_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DatasetGetDownloadURLResponse:
        """
        Get a download url for a Dataset.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/2/auto_eval/dataset/get_download_url",
            body=await async_maybe_transform(
                {"dataset_id": dataset_id}, dataset_get_download_url_params.DatasetGetDownloadURLParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetGetDownloadURLResponse,
        )

    async def get_view(
        self,
        *,
        dataset_file_id: str,
        dataset_id: str,
        filters: Iterable[dataset_get_view_params.Filter],
        after: int | NotGiven = NOT_GIVEN,
        get_last_page: bool | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        next_page_cursor: str | NotGiven = NOT_GIVEN,
        order_by: str | NotGiven = NOT_GIVEN,
        order_direction: str | NotGiven = NOT_GIVEN,
        previous_page_cursor: str | NotGiven = NOT_GIVEN,
        use_datasets_service: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DatasetGetViewResponse:
        """
        Get a paginated view of the data within a Dataset.

        Args:
          dataset_file_id: The ID of the (pinned) dataset file from which to retrieve content. Requests
              iterating over pages of results are recommended to use this pinned identifier
              after the first page in order to prevent any effects from a dataset changing
              between the queries.

          dataset_id: The ID of the dataset from which to retrieve content. When specified, gets data
              from the current file in the dataset.

          after: Pagination: The index, by row-order, after which to query results.

          limit: Pagination: The maximum number of results to return on this page.

          next_page_cursor: A cursor for the next page in the pagination, if one exists.

          order_by: Column to order results by

          order_direction: Direction to order results ("asc" or "desc")

          previous_page_cursor: A cursor for the previous page in the pagination, if one exists.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/2/auto_eval/dataset/get_view",
            body=await async_maybe_transform(
                {
                    "dataset_file_id": dataset_file_id,
                    "dataset_id": dataset_id,
                    "filters": filters,
                    "after": after,
                    "get_last_page": get_last_page,
                    "limit": limit,
                    "next_page_cursor": next_page_cursor,
                    "order_by": order_by,
                    "order_direction": order_direction,
                    "previous_page_cursor": previous_page_cursor,
                    "use_datasets_service": use_datasets_service,
                },
                dataset_get_view_params.DatasetGetViewParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetGetViewResponse,
        )

    async def upload_file(
        self,
        *,
        dataset_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DatasetUploadFileResponse:
        """Initiate a file upload to a Dataset.

        Call FinalizeSingleDatasetFileUpload to
        complete the upload with the presigned URL returned from this call.

        Args:
          dataset_id: The ID of the dataset corresponding to the file.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/2/auto_eval/dataset/upload_file",
            body=await async_maybe_transform(
                {"dataset_id": dataset_id}, dataset_upload_file_params.DatasetUploadFileParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatasetUploadFileResponse,
        )


class DatasetsResourceWithRawResponse:
    def __init__(self, datasets: DatasetsResource) -> None:
        self._datasets = datasets

        self.create = to_raw_response_wrapper(
            datasets.create,
        )
        self.list = to_raw_response_wrapper(
            datasets.list,
        )
        self.delete = to_raw_response_wrapper(
            datasets.delete,
        )
        self.copy = to_raw_response_wrapper(
            datasets.copy,
        )
        self.finalize_file_upload = to_raw_response_wrapper(
            datasets.finalize_file_upload,
        )
        self.get = to_raw_response_wrapper(
            datasets.get,
        )
        self.get_download_url = to_raw_response_wrapper(
            datasets.get_download_url,
        )
        self.get_view = to_raw_response_wrapper(
            datasets.get_view,
        )
        self.upload_file = to_raw_response_wrapper(
            datasets.upload_file,
        )


class AsyncDatasetsResourceWithRawResponse:
    def __init__(self, datasets: AsyncDatasetsResource) -> None:
        self._datasets = datasets

        self.create = async_to_raw_response_wrapper(
            datasets.create,
        )
        self.list = async_to_raw_response_wrapper(
            datasets.list,
        )
        self.delete = async_to_raw_response_wrapper(
            datasets.delete,
        )
        self.copy = async_to_raw_response_wrapper(
            datasets.copy,
        )
        self.finalize_file_upload = async_to_raw_response_wrapper(
            datasets.finalize_file_upload,
        )
        self.get = async_to_raw_response_wrapper(
            datasets.get,
        )
        self.get_download_url = async_to_raw_response_wrapper(
            datasets.get_download_url,
        )
        self.get_view = async_to_raw_response_wrapper(
            datasets.get_view,
        )
        self.upload_file = async_to_raw_response_wrapper(
            datasets.upload_file,
        )


class DatasetsResourceWithStreamingResponse:
    def __init__(self, datasets: DatasetsResource) -> None:
        self._datasets = datasets

        self.create = to_streamed_response_wrapper(
            datasets.create,
        )
        self.list = to_streamed_response_wrapper(
            datasets.list,
        )
        self.delete = to_streamed_response_wrapper(
            datasets.delete,
        )
        self.copy = to_streamed_response_wrapper(
            datasets.copy,
        )
        self.finalize_file_upload = to_streamed_response_wrapper(
            datasets.finalize_file_upload,
        )
        self.get = to_streamed_response_wrapper(
            datasets.get,
        )
        self.get_download_url = to_streamed_response_wrapper(
            datasets.get_download_url,
        )
        self.get_view = to_streamed_response_wrapper(
            datasets.get_view,
        )
        self.upload_file = to_streamed_response_wrapper(
            datasets.upload_file,
        )


class AsyncDatasetsResourceWithStreamingResponse:
    def __init__(self, datasets: AsyncDatasetsResource) -> None:
        self._datasets = datasets

        self.create = async_to_streamed_response_wrapper(
            datasets.create,
        )
        self.list = async_to_streamed_response_wrapper(
            datasets.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            datasets.delete,
        )
        self.copy = async_to_streamed_response_wrapper(
            datasets.copy,
        )
        self.finalize_file_upload = async_to_streamed_response_wrapper(
            datasets.finalize_file_upload,
        )
        self.get = async_to_streamed_response_wrapper(
            datasets.get,
        )
        self.get_download_url = async_to_streamed_response_wrapper(
            datasets.get_download_url,
        )
        self.get_view = async_to_streamed_response_wrapper(
            datasets.get_view,
        )
        self.upload_file = async_to_streamed_response_wrapper(
            datasets.upload_file,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ProjectDefaultParams"]


class ProjectDefaultParams(TypedDict, total=False):
    organization_id: Annotated[str, PropertyInfo(alias="organizationId")]
    """
    If provided, will get the default project for the organization Otherwise, will
    retrieve the default project for the user or organization associated with the
    api key used
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from lastmile import Lastmile, AsyncLastmile
from tests.utils import assert_matches_type
from lastmile.types import (
    LabelDatasetJobCreateResponse,
    LabelDatasetJobSubmitResponse,
    LabelDatasetJobGetStatusResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestLabelDatasetJobs:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Lastmile) -> None:
        label_dataset_job = client.label_dataset_jobs.create(
            pseudo_label_job_config={
                "base_evaluation_metric": "BASE_EVALUATION_METRIC_UNSPECIFIED",
                "dataset_id": "datasetId",
                "prompt_template": {
                    "id": "id",
                    "template": "template",
                },
                "selected_columns": ["string"],
                "skip_active_labeling": True,
            },
        )
        assert_matches_type(LabelDatasetJobCreateResponse, label_dataset_job, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Lastmile) -> None:
        label_dataset_job = client.label_dataset_jobs.create(
            pseudo_label_job_config={
                "base_evaluation_metric": "BASE_EVALUATION_METRIC_UNSPECIFIED",
                "dataset_id": "datasetId",
                "prompt_template": {
                    "id": "id",
                    "template": "template",
                },
                "selected_columns": ["string"],
                "skip_active_labeling": True,
                "active_labeled_dataset_id": "activeLabeledDatasetId",
                "description": "description",
                "few_shot_dataset_id": "fewShotDatasetId",
                "name": "name",
            },
        )
        assert_matches_type(LabelDatasetJobCreateResponse, label_dataset_job, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Lastmile) -> None:
        response = client.label_dataset_jobs.with_raw_response.create(
            pseudo_label_job_config={
                "base_evaluation_metric": "BASE_EVALUATION_METRIC_UNSPECIFIED",
                "dataset_id": "datasetId",
                "prompt_template": {
                    "id": "id",
                    "template": "template",
                },
                "selected_columns": ["string"],
                "skip_active_labeling": True,
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        label_dataset_job = response.parse()
        assert_matches_type(LabelDatasetJobCreateResponse, label_dataset_job, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Lastmile) -> None:
        with client.label_dataset_jobs.with_streaming_response.create(
            pseudo_label_job_config={
                "base_evaluation_metric": "BASE_EVALUATION_METRIC_UNSPECIFIED",
                "dataset_id": "datasetId",
                "prompt_template": {
                    "id": "id",
                    "template": "template",
                },
                "selected_columns": ["string"],
                "skip_active_labeling": True,
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            label_dataset_job = response.parse()
            assert_matches_type(LabelDatasetJobCreateResponse, label_dataset_job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_get_status(self, client: Lastmile) -> None:
        label_dataset_job = client.label_dataset_jobs.get_status(
            job_id="jobId",
        )
        assert_matches_type(LabelDatasetJobGetStatusResponse, label_dataset_job, path=["response"])

    @parametrize
    def test_raw_response_get_status(self, client: Lastmile) -> None:
        response = client.label_dataset_jobs.with_raw_response.get_status(
            job_id="jobId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        label_dataset_job = response.parse()
        assert_matches_type(LabelDatasetJobGetStatusResponse, label_dataset_job, path=["response"])

    @parametrize
    def test_streaming_response_get_status(self, client: Lastmile) -> None:
        with client.label_dataset_jobs.with_streaming_response.get_status(
            job_id="jobId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            label_dataset_job = response.parse()
            assert_matches_type(LabelDatasetJobGetStatusResponse, label_dataset_job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_submit(self, client: Lastmile) -> None:
        label_dataset_job = client.label_dataset_jobs.submit(
            job_id="jobId",
            pseudo_label_job_config={
                "base_evaluation_metric": "BASE_EVALUATION_METRIC_UNSPECIFIED",
                "dataset_id": "datasetId",
                "prompt_template": {
                    "id": "id",
                    "template": "template",
                },
                "selected_columns": ["string"],
                "skip_active_labeling": True,
            },
        )
        assert_matches_type(LabelDatasetJobSubmitResponse, label_dataset_job, path=["response"])

    @parametrize
    def test_method_submit_with_all_params(self, client: Lastmile) -> None:
        label_dataset_job = client.label_dataset_jobs.submit(
            job_id="jobId",
            pseudo_label_job_config={
                "base_evaluation_metric": "BASE_EVALUATION_METRIC_UNSPECIFIED",
                "dataset_id": "datasetId",
                "prompt_template": {
                    "id": "id",
                    "template": "template",
                },
                "selected_columns": ["string"],
                "skip_active_labeling": True,
                "active_labeled_dataset_id": "activeLabeledDatasetId",
                "description": "description",
                "few_shot_dataset_id": "fewShotDatasetId",
                "name": "name",
            },
        )
        assert_matches_type(LabelDatasetJobSubmitResponse, label_dataset_job, path=["response"])

    @parametrize
    def test_raw_response_submit(self, client: Lastmile) -> None:
        response = client.label_dataset_jobs.with_raw_response.submit(
            job_id="jobId",
            pseudo_label_job_config={
                "base_evaluation_metric": "BASE_EVALUATION_METRIC_UNSPECIFIED",
                "dataset_id": "datasetId",
                "prompt_template": {
                    "id": "id",
                    "template": "template",
                },
                "selected_columns": ["string"],
                "skip_active_labeling": True,
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        label_dataset_job = response.parse()
        assert_matches_type(LabelDatasetJobSubmitResponse, label_dataset_job, path=["response"])

    @parametrize
    def test_streaming_response_submit(self, client: Lastmile) -> None:
        with client.label_dataset_jobs.with_streaming_response.submit(
            job_id="jobId",
            pseudo_label_job_config={
                "base_evaluation_metric": "BASE_EVALUATION_METRIC_UNSPECIFIED",
                "dataset_id": "datasetId",
                "prompt_template": {
                    "id": "id",
                    "template": "template",
                },
                "selected_columns": ["string"],
                "skip_active_labeling": True,
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            label_dataset_job = response.parse()
            assert_matches_type(LabelDatasetJobSubmitResponse, label_dataset_job, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncLabelDatasetJobs:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncLastmile) -> None:
        label_dataset_job = await async_client.label_dataset_jobs.create(
            pseudo_label_job_config={
                "base_evaluation_metric": "BASE_EVALUATION_METRIC_UNSPECIFIED",
                "dataset_id": "datasetId",
                "prompt_template": {
                    "id": "id",
                    "template": "template",
                },
                "selected_columns": ["string"],
                "skip_active_labeling": True,
            },
        )
        assert_matches_type(LabelDatasetJobCreateResponse, label_dataset_job, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncLastmile) -> None:
        label_dataset_job = await async_client.label_dataset_jobs.create(
            pseudo_label_job_config={
                "base_evaluation_metric": "BASE_EVALUATION_METRIC_UNSPECIFIED",
                "dataset_id": "datasetId",
                "prompt_template": {
                    "id": "id",
                    "template": "template",
                },
                "selected_columns": ["string"],
                "skip_active_labeling": True,
                "active_labeled_dataset_id": "activeLabeledDatasetId",
                "description": "description",
                "few_shot_dataset_id": "fewShotDatasetId",
                "name": "name",
            },
        )
        assert_matches_type(LabelDatasetJobCreateResponse, label_dataset_job, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncLastmile) -> None:
        response = await async_client.label_dataset_jobs.with_raw_response.create(
            pseudo_label_job_config={
                "base_evaluation_metric": "BASE_EVALUATION_METRIC_UNSPECIFIED",
                "dataset_id": "datasetId",
                "prompt_template": {
                    "id": "id",
                    "template": "template",
                },
                "selected_columns": ["string"],
                "skip_active_labeling": True,
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        label_dataset_job = await response.parse()
        assert_matches_type(LabelDatasetJobCreateResponse, label_dataset_job, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncLastmile) -> None:
        async with async_client.label_dataset_jobs.with_streaming_response.create(
            pseudo_label_job_config={
                "base_evaluation_metric": "BASE_EVALUATION_METRIC_UNSPECIFIED",
                "dataset_id": "datasetId",
                "prompt_template": {
                    "id": "id",
                    "template": "template",
                },
                "selected_columns": ["string"],
                "skip_active_labeling": True,
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            label_dataset_job = await response.parse()
            assert_matches_type(LabelDatasetJobCreateResponse, label_dataset_job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_get_status(self, async_client: AsyncLastmile) -> None:
        label_dataset_job = await async_client.label_dataset_jobs.get_status(
            job_id="jobId",
        )
        assert_matches_type(LabelDatasetJobGetStatusResponse, label_dataset_job, path=["response"])

    @parametrize
    async def test_raw_response_get_status(self, async_client: AsyncLastmile) -> None:
        response = await async_client.label_dataset_jobs.with_raw_response.get_status(
            job_id="jobId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        label_dataset_job = await response.parse()
        assert_matches_type(LabelDatasetJobGetStatusResponse, label_dataset_job, path=["response"])

    @parametrize
    async def test_streaming_response_get_status(self, async_client: AsyncLastmile) -> None:
        async with async_client.label_dataset_jobs.with_streaming_response.get_status(
            job_id="jobId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            label_dataset_job = await response.parse()
            assert_matches_type(LabelDatasetJobGetStatusResponse, label_dataset_job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_submit(self, async_client: AsyncLastmile) -> None:
        label_dataset_job = await async_client.label_dataset_jobs.submit(
            job_id="jobId",
            pseudo_label_job_config={
                "base_evaluation_metric": "BASE_EVALUATION_METRIC_UNSPECIFIED",
                "dataset_id": "datasetId",
                "prompt_template": {
                    "id": "id",
                    "template": "template",
                },
                "selected_columns": ["string"],
                "skip_active_labeling": True,
            },
        )
        assert_matches_type(LabelDatasetJobSubmitResponse, label_dataset_job, path=["response"])

    @parametrize
    async def test_method_submit_with_all_params(self, async_client: AsyncLastmile) -> None:
        label_dataset_job = await async_client.label_dataset_jobs.submit(
            job_id="jobId",
            pseudo_label_job_config={
                "base_evaluation_metric": "BASE_EVALUATION_METRIC_UNSPECIFIED",
                "dataset_id": "datasetId",
                "prompt_template": {
                    "id": "id",
                    "template": "template",
                },
                "selected_columns": ["string"],
                "skip_active_labeling": True,
                "active_labeled_dataset_id": "activeLabeledDatasetId",
                "description": "description",
                "few_shot_dataset_id": "fewShotDatasetId",
                "name": "name",
            },
        )
        assert_matches_type(LabelDatasetJobSubmitResponse, label_dataset_job, path=["response"])

    @parametrize
    async def test_raw_response_submit(self, async_client: AsyncLastmile) -> None:
        response = await async_client.label_dataset_jobs.with_raw_response.submit(
            job_id="jobId",
            pseudo_label_job_config={
                "base_evaluation_metric": "BASE_EVALUATION_METRIC_UNSPECIFIED",
                "dataset_id": "datasetId",
                "prompt_template": {
                    "id": "id",
                    "template": "template",
                },
                "selected_columns": ["string"],
                "skip_active_labeling": True,
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        label_dataset_job = await response.parse()
        assert_matches_type(LabelDatasetJobSubmitResponse, label_dataset_job, path=["response"])

    @parametrize
    async def test_streaming_response_submit(self, async_client: AsyncLastmile) -> None:
        async with async_client.label_dataset_jobs.with_streaming_response.submit(
            job_id="jobId",
            pseudo_label_job_config={
                "base_evaluation_metric": "BASE_EVALUATION_METRIC_UNSPECIFIED",
                "dataset_id": "datasetId",
                "prompt_template": {
                    "id": "id",
                    "template": "template",
                },
                "selected_columns": ["string"],
                "skip_active_labeling": True,
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            label_dataset_job = await response.parse()
            assert_matches_type(LabelDatasetJobSubmitResponse, label_dataset_job, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from lastmile import Lastmile, AsyncLastmile
from tests.utils import assert_matches_type
from lastmile.types import (
    DatasetGetResponse,
    DatasetCopyResponse,
    DatasetListResponse,
    DatasetCreateResponse,
    DatasetDeleteResponse,
    DatasetGetViewResponse,
    DatasetUploadFileResponse,
    DatasetGetDownloadURLResponse,
    DatasetFinalizeFileUploadResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDatasets:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Lastmile) -> None:
        dataset = client.datasets.create()
        assert_matches_type(DatasetCreateResponse, dataset, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Lastmile) -> None:
        dataset = client.datasets.create(
            description="description",
            is_active_labels=True,
            is_few_shot_examples=True,
            name="name",
            project_id="projectId",
        )
        assert_matches_type(DatasetCreateResponse, dataset, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Lastmile) -> None:
        response = client.datasets.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(DatasetCreateResponse, dataset, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Lastmile) -> None:
        with client.datasets.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(DatasetCreateResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_list(self, client: Lastmile) -> None:
        dataset = client.datasets.list()
        assert_matches_type(DatasetListResponse, dataset, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Lastmile) -> None:
        dataset = client.datasets.list(
            filters={
                "project_id": "projectId",
                "query": "query",
            },
        )
        assert_matches_type(DatasetListResponse, dataset, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Lastmile) -> None:
        response = client.datasets.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(DatasetListResponse, dataset, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Lastmile) -> None:
        with client.datasets.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(DatasetListResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: Lastmile) -> None:
        dataset = client.datasets.delete(
            id="id",
        )
        assert_matches_type(DatasetDeleteResponse, dataset, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: Lastmile) -> None:
        response = client.datasets.with_raw_response.delete(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(DatasetDeleteResponse, dataset, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: Lastmile) -> None:
        with client.datasets.with_streaming_response.delete(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(DatasetDeleteResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_copy(self, client: Lastmile) -> None:
        dataset = client.datasets.copy(
            dataset_id="datasetId",
        )
        assert_matches_type(DatasetCopyResponse, dataset, path=["response"])

    @parametrize
    def test_method_copy_with_all_params(self, client: Lastmile) -> None:
        dataset = client.datasets.copy(
            dataset_id="datasetId",
            description="description",
            name="name",
            project_id="projectId",
        )
        assert_matches_type(DatasetCopyResponse, dataset, path=["response"])

    @parametrize
    def test_raw_response_copy(self, client: Lastmile) -> None:
        response = client.datasets.with_raw_response.copy(
            dataset_id="datasetId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(DatasetCopyResponse, dataset, path=["response"])

    @parametrize
    def test_streaming_response_copy(self, client: Lastmile) -> None:
        with client.datasets.with_streaming_response.copy(
            dataset_id="datasetId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(DatasetCopyResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_finalize_file_upload(self, client: Lastmile) -> None:
        dataset = client.datasets.finalize_file_upload(
            dataset_id="datasetId",
            s3_presigned_post={
                "fields": {"foo": "string"},
                "url": "url",
            },
        )
        assert_matches_type(DatasetFinalizeFileUploadResponse, dataset, path=["response"])

    @parametrize
    def test_raw_response_finalize_file_upload(self, client: Lastmile) -> None:
        response = client.datasets.with_raw_response.finalize_file_upload(
            dataset_id="datasetId",
            s3_presigned_post={
                "fields": {"foo": "string"},
                "url": "url",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(DatasetFinalizeFileUploadResponse, dataset, path=["response"])

    @parametrize
    def test_streaming_response_finalize_file_upload(self, client: Lastmile) -> None:
        with client.datasets.with_streaming_response.finalize_file_upload(
            dataset_id="datasetId",
            s3_presigned_post={
                "fields": {"foo": "string"},
                "url": "url",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(DatasetFinalizeFileUploadResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_get(self, client: Lastmile) -> None:
        dataset = client.datasets.get(
            id="id",
        )
        assert_matches_type(DatasetGetResponse, dataset, path=["response"])

    @parametrize
    def test_raw_response_get(self, client: Lastmile) -> None:
        response = client.datasets.with_raw_response.get(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(DatasetGetResponse, dataset, path=["response"])

    @parametrize
    def test_streaming_response_get(self, client: Lastmile) -> None:
        with client.datasets.with_streaming_response.get(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(DatasetGetResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_get_download_url(self, client: Lastmile) -> None:
        dataset = client.datasets.get_download_url(
            dataset_id="datasetId",
        )
        assert_matches_type(DatasetGetDownloadURLResponse, dataset, path=["response"])

    @parametrize
    def test_raw_response_get_download_url(self, client: Lastmile) -> None:
        response = client.datasets.with_raw_response.get_download_url(
            dataset_id="datasetId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(DatasetGetDownloadURLResponse, dataset, path=["response"])

    @parametrize
    def test_streaming_response_get_download_url(self, client: Lastmile) -> None:
        with client.datasets.with_streaming_response.get_download_url(
            dataset_id="datasetId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(DatasetGetDownloadURLResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_get_view(self, client: Lastmile) -> None:
        dataset = client.datasets.get_view(
            dataset_file_id="datasetFileId",
            dataset_id="datasetId",
            filters=[
                {
                    "all_of": {
                        "predicates": [
                            {
                                "column_name": "columnName",
                                "numeric_criteria": {
                                    "double_value": 0,
                                    "int64_value": 0,
                                    "operator": "OPERATOR_UNSPECIFIED",
                                },
                                "string_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "value": "value",
                                },
                                "string_list_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "values": ["string"],
                                },
                            }
                        ]
                    },
                    "any_of": {
                        "predicates": [
                            {
                                "column_name": "columnName",
                                "numeric_criteria": {
                                    "double_value": 0,
                                    "int64_value": 0,
                                    "operator": "OPERATOR_UNSPECIFIED",
                                },
                                "string_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "value": "value",
                                },
                                "string_list_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "values": ["string"],
                                },
                            }
                        ]
                    },
                }
            ],
        )
        assert_matches_type(DatasetGetViewResponse, dataset, path=["response"])

    @parametrize
    def test_method_get_view_with_all_params(self, client: Lastmile) -> None:
        dataset = client.datasets.get_view(
            dataset_file_id="datasetFileId",
            dataset_id="datasetId",
            filters=[
                {
                    "all_of": {
                        "predicates": [
                            {
                                "column_name": "columnName",
                                "numeric_criteria": {
                                    "double_value": 0,
                                    "int64_value": 0,
                                    "operator": "OPERATOR_UNSPECIFIED",
                                },
                                "string_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "value": "value",
                                },
                                "string_list_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "values": ["string"],
                                },
                            }
                        ]
                    },
                    "any_of": {
                        "predicates": [
                            {
                                "column_name": "columnName",
                                "numeric_criteria": {
                                    "double_value": 0,
                                    "int64_value": 0,
                                    "operator": "OPERATOR_UNSPECIFIED",
                                },
                                "string_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "value": "value",
                                },
                                "string_list_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "values": ["string"],
                                },
                            }
                        ]
                    },
                }
            ],
            after=0,
            get_last_page=True,
            limit=0,
            next_page_cursor="nextPageCursor",
            order_by="orderBy",
            order_direction="orderDirection",
            previous_page_cursor="previousPageCursor",
            use_datasets_service=True,
        )
        assert_matches_type(DatasetGetViewResponse, dataset, path=["response"])

    @parametrize
    def test_raw_response_get_view(self, client: Lastmile) -> None:
        response = client.datasets.with_raw_response.get_view(
            dataset_file_id="datasetFileId",
            dataset_id="datasetId",
            filters=[
                {
                    "all_of": {
                        "predicates": [
                            {
                                "column_name": "columnName",
                                "numeric_criteria": {
                                    "double_value": 0,
                                    "int64_value": 0,
                                    "operator": "OPERATOR_UNSPECIFIED",
                                },
                                "string_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "value": "value",
                                },
                                "string_list_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "values": ["string"],
                                },
                            }
                        ]
                    },
                    "any_of": {
                        "predicates": [
                            {
                                "column_name": "columnName",
                                "numeric_criteria": {
                                    "double_value": 0,
                                    "int64_value": 0,
                                    "operator": "OPERATOR_UNSPECIFIED",
                                },
                                "string_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "value": "value",
                                },
                                "string_list_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "values": ["string"],
                                },
                            }
                        ]
                    },
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(DatasetGetViewResponse, dataset, path=["response"])

    @parametrize
    def test_streaming_response_get_view(self, client: Lastmile) -> None:
        with client.datasets.with_streaming_response.get_view(
            dataset_file_id="datasetFileId",
            dataset_id="datasetId",
            filters=[
                {
                    "all_of": {
                        "predicates": [
                            {
                                "column_name": "columnName",
                                "numeric_criteria": {
                                    "double_value": 0,
                                    "int64_value": 0,
                                    "operator": "OPERATOR_UNSPECIFIED",
                                },
                                "string_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "value": "value",
                                },
                                "string_list_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "values": ["string"],
                                },
                            }
                        ]
                    },
                    "any_of": {
                        "predicates": [
                            {
                                "column_name": "columnName",
                                "numeric_criteria": {
                                    "double_value": 0,
                                    "int64_value": 0,
                                    "operator": "OPERATOR_UNSPECIFIED",
                                },
                                "string_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "value": "value",
                                },
                                "string_list_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "values": ["string"],
                                },
                            }
                        ]
                    },
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(DatasetGetViewResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_upload_file(self, client: Lastmile) -> None:
        dataset = client.datasets.upload_file(
            dataset_id="datasetId",
        )
        assert_matches_type(DatasetUploadFileResponse, dataset, path=["response"])

    @parametrize
    def test_raw_response_upload_file(self, client: Lastmile) -> None:
        response = client.datasets.with_raw_response.upload_file(
            dataset_id="datasetId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(DatasetUploadFileResponse, dataset, path=["response"])

    @parametrize
    def test_streaming_response_upload_file(self, client: Lastmile) -> None:
        with client.datasets.with_streaming_response.upload_file(
            dataset_id="datasetId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(DatasetUploadFileResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncDatasets:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncLastmile) -> None:
        dataset = await async_client.datasets.create()
        assert_matches_type(DatasetCreateResponse, dataset, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncLastmile) -> None:
        dataset = await async_client.datasets.create(
            description="description",
            is_active_labels=True,
            is_few_shot_examples=True,
            name="name",
            project_id="projectId",
        )
        assert_matches_type(DatasetCreateResponse, dataset, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncLastmile) -> None:
        response = await async_client.datasets.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(DatasetCreateResponse, dataset, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncLastmile) -> None:
        async with async_client.datasets.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(DatasetCreateResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_list(self, async_client: AsyncLastmile) -> None:
        dataset = await async_client.datasets.list()
        assert_matches_type(DatasetListResponse, dataset, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncLastmile) -> None:
        dataset = await async_client.datasets.list(
            filters={
                "project_id": "projectId",
                "query": "query",
            },
        )
        assert_matches_type(DatasetListResponse, dataset, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncLastmile) -> None:
        response = await async_client.datasets.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(DatasetListResponse, dataset, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncLastmile) -> None:
        async with async_client.datasets.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(DatasetListResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncLastmile) -> None:
        dataset = await async_client.datasets.delete(
            id="id",
        )
        assert_matches_type(DatasetDeleteResponse, dataset, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncLastmile) -> None:
        response = await async_client.datasets.with_raw_response.delete(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(DatasetDeleteResponse, dataset, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncLastmile) -> None:
        async with async_client.datasets.with_streaming_response.delete(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(DatasetDeleteResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_copy(self, async_client: AsyncLastmile) -> None:
        dataset = await async_client.datasets.copy(
            dataset_id="datasetId",
        )
        assert_matches_type(DatasetCopyResponse, dataset, path=["response"])

    @parametrize
    async def test_method_copy_with_all_params(self, async_client: AsyncLastmile) -> None:
        dataset = await async_client.datasets.copy(
            dataset_id="datasetId",
            description="description",
            name="name",
            project_id="projectId",
        )
        assert_matches_type(DatasetCopyResponse, dataset, path=["response"])

    @parametrize
    async def test_raw_response_copy(self, async_client: AsyncLastmile) -> None:
        response = await async_client.datasets.with_raw_response.copy(
            dataset_id="datasetId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(DatasetCopyResponse, dataset, path=["response"])

    @parametrize
    async def test_streaming_response_copy(self, async_client: AsyncLastmile) -> None:
        async with async_client.datasets.with_streaming_response.copy(
            dataset_id="datasetId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(DatasetCopyResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_finalize_file_upload(self, async_client: AsyncLastmile) -> None:
        dataset = await async_client.datasets.finalize_file_upload(
            dataset_id="datasetId",
            s3_presigned_post={
                "fields": {"foo": "string"},
                "url": "url",
            },
        )
        assert_matches_type(DatasetFinalizeFileUploadResponse, dataset, path=["response"])

    @parametrize
    async def test_raw_response_finalize_file_upload(self, async_client: AsyncLastmile) -> None:
        response = await async_client.datasets.with_raw_response.finalize_file_upload(
            dataset_id="datasetId",
            s3_presigned_post={
                "fields": {"foo": "string"},
                "url": "url",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(DatasetFinalizeFileUploadResponse, dataset, path=["response"])

    @parametrize
    async def test_streaming_response_finalize_file_upload(self, async_client: AsyncLastmile) -> None:
        async with async_client.datasets.with_streaming_response.finalize_file_upload(
            dataset_id="datasetId",
            s3_presigned_post={
                "fields": {"foo": "string"},
                "url": "url",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(DatasetFinalizeFileUploadResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_get(self, async_client: AsyncLastmile) -> None:
        dataset = await async_client.datasets.get(
            id="id",
        )
        assert_matches_type(DatasetGetResponse, dataset, path=["response"])

    @parametrize
    async def test_raw_response_get(self, async_client: AsyncLastmile) -> None:
        response = await async_client.datasets.with_raw_response.get(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(DatasetGetResponse, dataset, path=["response"])

    @parametrize
    async def test_streaming_response_get(self, async_client: AsyncLastmile) -> None:
        async with async_client.datasets.with_streaming_response.get(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(DatasetGetResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_get_download_url(self, async_client: AsyncLastmile) -> None:
        dataset = await async_client.datasets.get_download_url(
            dataset_id="datasetId",
        )
        assert_matches_type(DatasetGetDownloadURLResponse, dataset, path=["response"])

    @parametrize
    async def test_raw_response_get_download_url(self, async_client: AsyncLastmile) -> None:
        response = await async_client.datasets.with_raw_response.get_download_url(
            dataset_id="datasetId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(DatasetGetDownloadURLResponse, dataset, path=["response"])

    @parametrize
    async def test_streaming_response_get_download_url(self, async_client: AsyncLastmile) -> None:
        async with async_client.datasets.with_streaming_response.get_download_url(
            dataset_id="datasetId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(DatasetGetDownloadURLResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_get_view(self, async_client: AsyncLastmile) -> None:
        dataset = await async_client.datasets.get_view(
            dataset_file_id="datasetFileId",
            dataset_id="datasetId",
            filters=[
                {
                    "all_of": {
                        "predicates": [
                            {
                                "column_name": "columnName",
                                "numeric_criteria": {
                                    "double_value": 0,
                                    "int64_value": 0,
                                    "operator": "OPERATOR_UNSPECIFIED",
                                },
                                "string_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "value": "value",
                                },
                                "string_list_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "values": ["string"],
                                },
                            }
                        ]
                    },
                    "any_of": {
                        "predicates": [
                            {
                                "column_name": "columnName",
                                "numeric_criteria": {
                                    "double_value": 0,
                                    "int64_value": 0,
                                    "operator": "OPERATOR_UNSPECIFIED",
                                },
                                "string_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "value": "value",
                                },
                                "string_list_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "values": ["string"],
                                },
                            }
                        ]
                    },
                }
            ],
        )
        assert_matches_type(DatasetGetViewResponse, dataset, path=["response"])

    @parametrize
    async def test_method_get_view_with_all_params(self, async_client: AsyncLastmile) -> None:
        dataset = await async_client.datasets.get_view(
            dataset_file_id="datasetFileId",
            dataset_id="datasetId",
            filters=[
                {
                    "all_of": {
                        "predicates": [
                            {
                                "column_name": "columnName",
                                "numeric_criteria": {
                                    "double_value": 0,
                                    "int64_value": 0,
                                    "operator": "OPERATOR_UNSPECIFIED",
                                },
                                "string_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "value": "value",
                                },
                                "string_list_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "values": ["string"],
                                },
                            }
                        ]
                    },
                    "any_of": {
                        "predicates": [
                            {
                                "column_name": "columnName",
                                "numeric_criteria": {
                                    "double_value": 0,
                                    "int64_value": 0,
                                    "operator": "OPERATOR_UNSPECIFIED",
                                },
                                "string_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "value": "value",
                                },
                                "string_list_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "values": ["string"],
                                },
                            }
                        ]
                    },
                }
            ],
            after=0,
            get_last_page=True,
            limit=0,
            next_page_cursor="nextPageCursor",
            order_by="orderBy",
            order_direction="orderDirection",
            previous_page_cursor="previousPageCursor",
            use_datasets_service=True,
        )
        assert_matches_type(DatasetGetViewResponse, dataset, path=["response"])

    @parametrize
    async def test_raw_response_get_view(self, async_client: AsyncLastmile) -> None:
        response = await async_client.datasets.with_raw_response.get_view(
            dataset_file_id="datasetFileId",
            dataset_id="datasetId",
            filters=[
                {
                    "all_of": {
                        "predicates": [
                            {
                                "column_name": "columnName",
                                "numeric_criteria": {
                                    "double_value": 0,
                                    "int64_value": 0,
                                    "operator": "OPERATOR_UNSPECIFIED",
                                },
                                "string_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "value": "value",
                                },
                                "string_list_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "values": ["string"],
                                },
                            }
                        ]
                    },
                    "any_of": {
                        "predicates": [
                            {
                                "column_name": "columnName",
                                "numeric_criteria": {
                                    "double_value": 0,
                                    "int64_value": 0,
                                    "operator": "OPERATOR_UNSPECIFIED",
                                },
                                "string_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "value": "value",
                                },
                                "string_list_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "values": ["string"],
                                },
                            }
                        ]
                    },
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(DatasetGetViewResponse, dataset, path=["response"])

    @parametrize
    async def test_streaming_response_get_view(self, async_client: AsyncLastmile) -> None:
        async with async_client.datasets.with_streaming_response.get_view(
            dataset_file_id="datasetFileId",
            dataset_id="datasetId",
            filters=[
                {
                    "all_of": {
                        "predicates": [
                            {
                                "column_name": "columnName",
                                "numeric_criteria": {
                                    "double_value": 0,
                                    "int64_value": 0,
                                    "operator": "OPERATOR_UNSPECIFIED",
                                },
                                "string_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "value": "value",
                                },
                                "string_list_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "values": ["string"],
                                },
                            }
                        ]
                    },
                    "any_of": {
                        "predicates": [
                            {
                                "column_name": "columnName",
                                "numeric_criteria": {
                                    "double_value": 0,
                                    "int64_value": 0,
                                    "operator": "OPERATOR_UNSPECIFIED",
                                },
                                "string_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "value": "value",
                                },
                                "string_list_criteria": {
                                    "operator": "OPERATOR_UNSPECIFIED",
                                    "values": ["string"],
                                },
                            }
                        ]
                    },
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(DatasetGetViewResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_upload_file(self, async_client: AsyncLastmile) -> None:
        dataset = await async_client.datasets.upload_file(
            dataset_id="datasetId",
        )
        assert_matches_type(DatasetUploadFileResponse, dataset, path=["response"])

    @parametrize
    async def test_raw_response_upload_file(self, async_client: AsyncLastmile) -> None:
        response = await async_client.datasets.with_raw_response.upload_file(
            dataset_id="datasetId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(DatasetUploadFileResponse, dataset, path=["response"])

    @parametrize
    async def test_streaming_response_upload_file(self, async_client: AsyncLastmile) -> None:
        async with async_client.datasets.with_streaming_response.upload_file(
            dataset_id="datasetId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(DatasetUploadFileResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ExperimentListParams", "Filters"]


class ExperimentListParams(TypedDict, total=False):
    filters: Filters

    page_index: Annotated[int, PropertyInfo(alias="pageIndex")]

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]


class Filters(TypedDict, total=False):
    project_id: Annotated[str, PropertyInfo(alias="projectId")]
    """search only experiments associated with specific project"""

    query: str
    """search query substring match for name and description"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ExperimentCreateParams", "Metadata"]


class ExperimentCreateParams(TypedDict, total=False):
    name: Required[str]
    """Human-readable name for the experiment"""

    project_id: Required[Annotated[str, PropertyInfo(alias="projectId")]]
    """Project the experiment is associated with"""

    description: str
    """Succint description of the experiment, if one exists."""

    metadata: Metadata
    """
    Metadata relevant to the application configuration from which all experiment
    evaluation runs are derived. E.g. 'llm_model', 'chunk_size'
    """


class Metadata(TypedDict, total=False):
    fields: Required[Dict[str, Dict[str, object]]]
